==> operation-csv-1.1.1-20140324.153943-1-sources.jar: org/gcube/data/analysis/tabulardata/operation/csv/exporter/CSVExport.java <==
package org.gcube.data.analysis.tabulardata.operation.csv.exporter;

import static org.gcube.data.analysis.tabulardata.operation.csv.Constants.COLUMNS;
import static org.gcube.data.analysis.tabulardata.operation.csv.Constants.ENCODING;
import static org.gcube.data.analysis.tabulardata.operation.csv.Constants.SEPARATOR;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.gcube.contentmanagement.blobstorage.service.IClient;
import org.gcube.contentmanager.storageclient.wrapper.AccessType;
import org.gcube.contentmanager.storageclient.wrapper.MemoryType;
import org.gcube.contentmanager.storageclient.wrapper.StorageClient;
import org.gcube.data.analysis.tabulardata.cube.CubeManager;
import org.gcube.data.analysis.tabulardata.cube.data.connection.DatabaseConnectionProvider;
import org.gcube.data.analysis.tabulardata.cube.tablemanagers.TableMetaCreator;
import org.gcube.data.analysis.tabulardata.model.column.Column;
import org.gcube.data.analysis.tabulardata.model.metadata.common.NamesMetadata;
import org.gcube.data.analysis.tabulardata.model.metadata.table.ExportMetadata;
import org.gcube.data.analysis.tabulardata.model.table.Table;
import org.gcube.data.analysis.tabulardata.operation.csv.Constants;
import org.gcube.data.analysis.tabulardata.operation.invocation.OperationInvocation;
import org.gcube.data.analysis.tabulardata.operation.worker.ImmutableWorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.Worker;
import org.gcube.data.analysis.tabulardata.operation.worker.WorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.exceptions.WorkerException;
import org.postgresql.PGConnection;
import org.postgresql.copy.CopyManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CSVExport extends Worker {
	
	private static Logger logger = LoggerFactory.getLogger(CSVExport.class);
	
	CubeManager cubeManager;
	
	DatabaseConnectionProvider connectionProvider;
	
	private String encoding;
	private Character separator;
	private List<String> selectedColumns;
	
	public CSVExport(OperationInvocation invocation, CubeManager cubeManager,
			DatabaseConnectionProvider connectionProvider) {
		super(invocation);
		retrieveParameters();
		this.cubeManager = cubeManager;
		this.connectionProvider = connectionProvider;
	}

	public WorkerResult execute() throws WorkerException {
		Table table = cubeManager.getTable(getSourceInvocation().getTargetTableId());
		updateProgress(0.1f);
		File exportFile;
		try{
			exportFile = File.createTempFile("export", ".csv");
		}catch (Exception e) {
			logger.error("error creating file", e);
			throw new WorkerException("error creating file", e);
		}
		updateProgress(0.3f);
		OutputStreamWriter outputStreamWriter = null;
		try{
			outputStreamWriter = new OutputStreamWriter(new FileOutputStream(exportFile));
			long updated = copy(outputStreamWriter, table, getSourceInvocation().getParameterInstances());
			logger.debug("exported "+updated+" entries");
		}catch (Exception e) {
			logger.error("error copying table to file", e);
			exportFile.delete();
			throw new WorkerException("error copying table to file", e);
		}finally{
			if (outputStreamWriter!=null)
				try {
					outputStreamWriter.close();
				} catch (IOException e) {logger.warn("error closing output stream",e);}
		}
		updateProgress(0.7f);
		String url;
		try{
			url = storeFile(exportFile);
		}catch (Exception e) {
			logger.error("error storing file", e);
			throw new WorkerException("error storing file", e);
		}finally{
			exportFile.delete();
		}
		updateProgress(0.9f);
		Table returnTable = createMetaTable(table, url);
		return new ImmutableWorkerResult(returnTable);
	}

	private String storeFile(File exportFile){
		IClient client = new StorageClient(Constants.SERVICE_CLASS, Constants.SERVICE_NAME, "CSVExport", AccessType.SHARED, MemoryType.VOLATILE).getClient();
		String remotePath = "/CSVexport/"+exportFile.getName();
		client.put(true).LFile(exportFile.getAbsolutePath()).RFile(remotePath);
		return  client.getUrl().RFile(remotePath);
	}

	private Table createMetaTable(Table table, String url) {
		TableMetaCreator tmc = cubeManager.modifyTableMeta(table.getId());
		tmc.setTableMetadata(new ExportMetadata("CSV", url, new Date()));
		return tmc.create();
	}
	
	private long copy(OutputStreamWriter outputStreamWriter, Table table, Map<String, Object> parameters) throws Exception {
		PGConnection conn = connectionProvider.getPostgreSQLConnection();
		CopyManager cpManager = conn.getCopyAPI();
		StringBuilder columns = new StringBuilder("SELECT ");
		
		for (Column c : table.getColumns()) {
			if(selectedColumns.contains(c.getName())){
				String label = c.getName();
				try{
					NamesMetadata metadata = ((NamesMetadata)c.getMetadata(NamesMetadata.class));
					if (metadata.hasTextWithLocale("en"))
						label = metadata.getTextWithLocale("en").getValue();
				}catch (Exception e) {}
				if (label.isEmpty()) 
					label= c.getLocalId().getValue();
				columns.append(c.getName()).append(" as ").append(String.format("\"%s\"", label));
				columns.append(",");
			}
		}
		columns.deleteCharAt(columns.length() - 1);
		columns.append(" from "+table.getName());
		
		String sqlCmd = String.format("COPY ( %s ) TO STDOUT ( FORMAT CSV ,DELIMITER '%c', HEADER %b, ENCODING '%s');",
				columns.toString(), separator, true, encoding);
		logger.info("executing copy for csv import with query {}",sqlCmd);
		
		return cpManager.copyOut(sqlCmd, outputStreamWriter);
	}
	
	@SuppressWarnings("unchecked")
	private void retrieveParameters() {
		Map<String, Object> parameters = getSourceInvocation().getParameterInstances();
		separator = ((String) parameters.get(SEPARATOR)).charAt(0);
		encoding = (String) parameters.get(ENCODING);
		
		selectedColumns =new ArrayList<String>();
		Object columnsParam=parameters.get(COLUMNS);
		if(columnsParam instanceof Iterable<?>)
			for(String col:(Iterable<String>)columnsParam)selectedColumns.add(col);
		else selectedColumns.add((String)columnsParam);
	}
}

==> operation-csv-1.1.2-20140522.153232-19-sources.jar: org/gcube/data/analysis/tabulardata/operation/csv/exporter/CSVExport.java <==
package org.gcube.data.analysis.tabulardata.operation.csv.exporter;

import static org.gcube.data.analysis.tabulardata.operation.csv.Constants.COLUMNS;
import static org.gcube.data.analysis.tabulardata.operation.csv.Constants.ENCODING;
import static org.gcube.data.analysis.tabulardata.operation.csv.Constants.SEPARATOR;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

import org.gcube.contentmanagement.blobstorage.service.IClient;
import org.gcube.contentmanager.storageclient.wrapper.AccessType;
import org.gcube.contentmanager.storageclient.wrapper.MemoryType;
import org.gcube.contentmanager.storageclient.wrapper.StorageClient;
import org.gcube.data.analysis.tabulardata.cube.CubeManager;
import org.gcube.data.analysis.tabulardata.cube.data.connection.DatabaseConnectionProvider;
import org.gcube.data.analysis.tabulardata.model.column.Column;
import org.gcube.data.analysis.tabulardata.model.metadata.common.NamesMetadata;
import org.gcube.data.analysis.tabulardata.model.table.Table;
import org.gcube.data.analysis.tabulardata.operation.csv.Constants;
import org.gcube.data.analysis.tabulardata.operation.invocation.OperationInvocation;
import org.gcube.data.analysis.tabulardata.operation.worker.exceptions.WorkerException;
import org.gcube.data.analysis.tabulardata.operation.worker.results.resources.ImmutableURIResult;
import org.gcube.data.analysis.tabulardata.operation.worker.results.resources.ResourceDescriptorResult;
import org.gcube.data.analysis.tabulardata.operation.worker.types.ResourceCreatorWorker;
import org.postgresql.PGConnection;
import org.postgresql.copy.CopyManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CSVExport extends ResourceCreatorWorker {
	
	private static Logger logger = LoggerFactory.getLogger(CSVExport.class);
	
	CubeManager cubeManager;
	
	DatabaseConnectionProvider connectionProvider;
	
	private String encoding;
	private Character separator;
	private List<String> selectedColumns;
	
	public CSVExport(OperationInvocation invocation, CubeManager cubeManager,
			DatabaseConnectionProvider connectionProvider) {
		super(invocation);
		retrieveParameters();
		this.cubeManager = cubeManager;
		this.connectionProvider = connectionProvider;
	}

	public ResourceDescriptorResult execute() throws WorkerException {
		Table table = cubeManager.getTable(getSourceInvocation().getTargetTableId());
		updateProgress(0.1f,"Creating export file");
		File exportFile;
		try{
			exportFile = File.createTempFile("export", ".csv");
		}catch (Exception e) {
			logger.error("error creating file", e);
			throw new WorkerException("error creating file", e);
		}
		updateProgress(0.3f,"Writing file with data");
		OutputStreamWriter outputStreamWriter = null;
		try{
			outputStreamWriter = new OutputStreamWriter(new FileOutputStream(exportFile));
			long updated = copy(outputStreamWriter, table, getSourceInvocation().getParameterInstances());
			logger.debug("exported "+updated+" entries");
		}catch (Exception e) {
			logger.error("error copying table to file", e);
			exportFile.delete();
			throw new WorkerException("error copying table to file", e);
		}finally{
			if (outputStreamWriter!=null)
				try {
					outputStreamWriter.close();
				} catch (IOException e) {logger.warn("error closing output stream",e);}
		}
		updateProgress(0.7f,"Storing file on storage");
		String url;
		try{
			url = storeFile(exportFile);
		}catch (Exception e) {
			logger.error("error storing file", e);
			throw new WorkerException("error storing file", e);
		}finally{
			exportFile.delete();
		}
		updateProgress(0.9f,"Finalizing");
		return createDescriptor(url);
	}

	private String storeFile(File exportFile){
		IClient client = new StorageClient(Constants.SERVICE_CLASS, Constants.SERVICE_NAME, "CSVExport", AccessType.SHARED, MemoryType.VOLATILE).getClient();
		String remotePath = "/CSVexport/"+exportFile.getName();
		client.put(true).LFile(exportFile.getAbsolutePath()).RFile(remotePath);
		return  client.getUrl().RFile(remotePath);
	}

	private ResourceDescriptorResult createDescriptor(String url) throws WorkerException {
		try {
			return new ImmutableURIResult(new URI(url), "csv exported file", Calendar.getInstance());
		} catch (URISyntaxException e) {
			throw new WorkerException("exported url not valid", e);
		}
	}
	
	private long copy(OutputStreamWriter outputStreamWriter, Table table, Map<String, Object> parameters) throws Exception {
		PGConnection conn = connectionProvider.getPostgreSQLConnection();
		CopyManager cpManager = conn.getCopyAPI();
		StringBuilder columns = new StringBuilder("SELECT ");
		
		for (Column c : table.getColumns()) {
			if(selectedColumns.contains(c.getName())){
				String label = c.getName();
				try{
					NamesMetadata metadata = ((NamesMetadata)c.getMetadata(NamesMetadata.class));
					if (metadata.hasTextWithLocale("en"))
						label = metadata.getTextWithLocale("en").getValue();
				}catch (Exception e) {}
				if (label.isEmpty()) 
					label= c.getLocalId().getValue();
				columns.append(c.getName()).append(" as ").append(String.format("\"%s\"", label));
				columns.append(",");
			}
		}
		columns.deleteCharAt(columns.length() - 1);
		columns.append(" from "+table.getName());
		
		String sqlCmd = String.format("COPY ( %s ) TO STDOUT ( FORMAT CSV ,DELIMITER '%c', HEADER %b, ENCODING '%s');",
				columns.toString(), separator, true, encoding);
		logger.info("executing copy for csv import with query {}",sqlCmd);
		
		return cpManager.copyOut(sqlCmd, outputStreamWriter);
	}
	
	@SuppressWarnings("unchecked")
	private void retrieveParameters() {
		Map<String, Object> parameters = getSourceInvocation().getParameterInstances();
		separator = ((String) parameters.get(SEPARATOR)).charAt(0);
		encoding = (String) parameters.get(ENCODING);
		
		selectedColumns =new ArrayList<String>();
		Object columnsParam=parameters.get(COLUMNS);
		if(columnsParam instanceof Iterable<?>)
			for(String col:(Iterable<String>)columnsParam)selectedColumns.add(col);
		else selectedColumns.add((String)columnsParam);
	}
}
